import asyncio
import websockets
import urllib.parse

UNITY_ID = "unity"
LIB_ID = "lib"

PUSH = "push"
PULL = "pull"

unity_frame = None
height_map = None

async def echo(websocket):
    global unity_frame, height_map

    query_params = urllib.parse.parse_qs(urllib.parse.urlparse(websocket.request.path).query)
    client_id = query_params.get('client_id', ['Unknown'])[0]
    operation = query_params.get('operation', ['Unknown'])[0]

    try:
        async for message in websocket:
            print(f"Received: from {client_id} with operation {operation}")

            # Unity
            if client_id == UNITY_ID:
                if operation == PUSH:
                    unity_frame = message
                    await websocket.send("200: OK")
                elif operation == PULL:
                    await websocket.send(height_map)
                else:
                    await websocket.send("400: Bad Request")

            # Lib
            elif client_id == LIB_ID:
                if operation == PUSH:
                    height_map = message
                elif operation == PULL:
                    await websocket.send(unity_frame)
                else:
                    await websocket.send("400: Bad Request")

            # Bad url ?
            else:
                await websocket.send("403: Unable to identify you")
            
    except websockets.exceptions.ConnectionClosed as e:
        print(f"Client disconnected: {e.code}")
    finally:
        print("Connection closed")

async def main():
    async with websockets.serve(echo, "localhost", 8765, max_size=10**7):
        print("WebSocket server running on ws://localhost:8765")
        await asyncio.Future()  # Run the server indefinitely

asyncio.run(main())
